#include <Python.h>

#include "cbarprivate.h"

#include <qpushbutton.h>
#include <qmessagebox.h>
#include <kstandarddirs.h>
#include <kconfig.h>
#include <kdebug.h>

/************************* CbarWidgetPrivate *********************/

CbarWidgetPrivate::CbarWidgetPrivate(
    CbarWidget *_parent,
    QString const & _cbarName
    ):
    parent(_parent),
    cbarName(_cbarName),
    cbarBinding(new CbarBinding(parent)),
    scriptLoader(NULL)
{
}

CbarWidgetPrivate::~CbarWidgetPrivate()
{
    delete cbarBinding;
}

bool CbarWidgetPrivate::executeScript(
    QString const & scriptFileName
    )
{
    QTime startTime = QTime::currentTime();
    startTime.start();
    kdDebug() << "CbarWidgetPrivate::executeScript(): Starting running script: " << scriptFileName 
        << ", start time=" << startTime.toString("hh:mm:ss.zzz")  << " \n";
    
    Script::Ptr scriptPtr = Script::Ptr(
        new Script()
        );
        
    this->scripts.insert(scriptFileName, scriptPtr, true);
    
/*    action->activate();
    bool ok = action->hadException();*/
    FILE *file = fopen(scriptFileName.data(), "r");
    
    if (file == NULL)
    {
        kdDebug() << "Failed to open file " << scriptFileName << ", error=" << errno << " (" << strerror(errno) << ")\n"; //FIXME: remove strerror, it is not thread-safe
        return false;
    }
    
    bool ok = PyRun_SimpleFile(file, scriptFileName.data());
    
    fclose(file);
    
    QTime endTime = QTime::currentTime();
    kdDebug() << "CbarWidgetPrivate::executeScript(): Finished running script: " << scriptFileName 
        << ", start time=" << startTime.toString("hh:mm:ss.zzz")
        << ", endTime=" << endTime.toString("hh:mm:ss.zzz")
        << ", duration=" <<  startTime.elapsed() << " ms"
        <<  " \n";
        
    return ok;
}

void CbarWidgetPrivate::themeManagerDestroyed()
{
    kdDebug() << "CbarWidgetPrivate::themeManagerDestroyed()\n";
    QMessageBox::warning(NULL, "themeManagerDestroyed", "Theme manager destroyed");
}

CbarWidgetPrivate::Script::Script(
    )
{
}

KConfig *CbarWidgetPrivate::getConfig()
{
    return new KConfig(this->cbarName + "/cbarrc");
}

void CbarWidgetPrivate::loadScriptLoader()
{
    KConfig *config = this->getConfig();
    
    config->setGroup("ScriptLoader");
    QString scriptLoaderPath = config->readPathEntry("scriptPath");
    kdDebug() << "scriptLoaderPath=" << scriptLoaderPath << "\n";
    
    if (scriptLoaderPath.isEmpty() == true)
    {
        kdDebug() << "Cannot find script loader config file, trying default\n";
        scriptLoaderPath = ::locate("data", "cbar/scripts/defaultScriptLoader.py");
    }
    
    kdDebug() << "Executing script loader script, scriptLoaderPath=" << scriptLoaderPath << "\n";
    this->executeScript(scriptLoaderPath);
    
    delete config;
}



#include "cbarprivate.moc"
