#ifndef __CBAR_PRIVATE_H__
#define __CBAR_PRIVATE_H__

#include <qobject.h>
#include <ksharedptr.h>
#include <qmap.h>

class QPushButton;
class CbarWidget;
class KConfig;

class CbarBinding
{
    public:
        explicit CbarBinding(CbarWidget *widget);
        virtual ~CbarBinding();
};

class CbarWidgetPrivate:
    public QObject,
    public KShared
{
    Q_OBJECT;
public:
    CbarWidgetPrivate(
        CbarWidget *parent,
        QString const & cbarName
        );
    virtual ~CbarWidgetPrivate();
    
public slots:
    bool executeScript(
        QString const & scriptFileName
        );
    void themeManagerDestroyed();
protected:
    CbarWidget *parent;
    QString cbarName;
    CbarBinding *cbarBinding;
    QObject *scriptLoader;
    
    class Script:
        public KShared
    {
    public:
        typedef KSharedPtr<Script> Ptr;
        
        Script();
    };
    
    typedef QMap<QString, Script::Ptr> ScriptsContainer;
    ScriptsContainer scripts;
    
    /**
        Loads config for this instance of Cbar.
    */
    KConfig *getConfig();

    /**
        Loads script loader script.
    */
    void loadScriptLoader();

    
    friend class CbarWidget;
};

#endif //#ifndef __CBAR_PRIVATE_H__
