import qt
from qt import *
import cbar
import kdecore
import kfile

def getConfig():
    cbarInterface = cbar.Cbar()
    cbarName = cbarInterface.getCbarName()
    
    kconfig = kdecore.KConfig(cbarName + "/cbarrc")
    return kconfig

def getAvailableScripts(scriptTypeFilter, enabledConfigGroup, enabledConfigEntryName):
    global getConfig
    #Desktop files handling copied from Konqueror sidebar
    s = ""
    desktopFileNames = kdecore.KGlobal.dirs().findAllResources(
        "data",
       "cbar/scripts/*.desktop"
       )
    cbarConfig = getConfig()
    cbarConfig.setGroup(enabledConfigGroup)
    enabledScriptsList = cbarConfig.readPathListEntry(enabledConfigEntryName) #entryName, default
    enabledScripts = {}
    for scriptPath in enabledScriptsList:
        enabledScripts[str(scriptPath)] = 1
        
    scripts = []
    
    for desktopFileName in desktopFileNames:
        scriptParams = {}
        scriptDesktopFile = kdecore.KSimpleConfig(desktopFileName)
        scriptDesktopFile.setGroup("Desktop Entry")
        scriptType = str(scriptDesktopFile.readEntry("X-KDE-Cbar-Script-Type"))
        if scriptType != scriptTypeFilter:
            continue
        scriptParams["name"] = str(scriptDesktopFile.readEntry("Name"))
        scriptBaseName = str(scriptDesktopFile.readEntry("X-KDE-Cbar-Script"))
        if scriptBaseName[0] == "/":
            scriptParams["path"] = str(scriptBaseName)
        else:
            scriptParams["path"] = str(kdecore.locate("data", "cbar/scripts/" + str(scriptBaseName)))
        scriptParams["description"] = str(scriptDesktopFile.readEntry("Comment"))
        
        if enabledScripts.has_key(scriptParams["path"]):
            scriptParams["enabled"] = True
        else:
            scriptParams["enabled"] = False
            
        scripts.append(scriptParams)
    return (scripts, enabledScriptsList)

class ScriptListViewItem(qt.QListViewItem):
    def __init__(self, scriptParams, *args):
        apply(qt.QListViewItem.__init__, (self,) + args)
        self.scriptParams = scriptParams
        self.updateTexts()
    def getParams(self):
        return self.scriptParams
    def enabledText(self, value):
        if value == True:
            return "Enabled"
        else:
            return "Disabled"
        
    def updateTexts(self):
        self.setText(0, self.scriptParams["name"])
        self.setText(1, self.enabledText(self.scriptParams["enabled"]))

class WidgetScriptsWindow(qt.QWidget):
    def __init__(self,parent = None,name = None):
        qt.QWidget.__init__(self,parent,name)

        Form1Layout = qt.QGridLayout(self,1,1,11,6,"Form1Layout")

        layout4 = qt.QVBoxLayout(None,0,6,"layout4")

        self.scriptsList = qt.QListView(self,"scriptsList")
        self.scriptsList.addColumn("Script")
        self.scriptsList.addColumn("Enabled")
        self.scriptsList.setResizePolicy(qt.QListView.AutoOneFit)
        self.scriptsList.setShowSortIndicator(1)
        layout4.addWidget(self.scriptsList)

        layout2 = qt.QHBoxLayout(None,0,6,"layout2")

        self.textLabel1 = qt.QLabel(self,"textLabel1")
        self.textLabel1.setText("Script name:")
        layout2.addWidget(self.textLabel1)

        self.scriptNameField = qt.QLabel(self,"scriptNameField")
        layout2.addWidget(self.scriptNameField)
        layout4.addLayout(layout2)

        layout2_2 = qt.QHBoxLayout(None,0,6,"layout2_2")

        self.textLabel1_2 = qt.QLabel(self,"textLabel1_2")
        self.textLabel1_2.setText("Script path:")
        layout2_2.addWidget(self.textLabel1_2)

        self.scriptPathField = qt.QLabel(self,"scriptPathField")
        layout2_2.addWidget(self.scriptPathField)
        layout4.addLayout(layout2_2)

        self.textLabel3 = qt.QLabel(self,"textLabel3")
        self.textLabel3.setText("Script description:")
        layout4.addWidget(self.textLabel3)

        self.scriptDescriptionField = qt.QTextBrowser(self,"scriptDescriptionField")
        self.scriptDescriptionField.setMinimumSize(qt.QSize(60,0))
        layout4.addWidget(self.scriptDescriptionField)

        Form1Layout.addMultiCellLayout(layout4,0,1,0,0)

        self.enableToggleButton = qt.QPushButton(self,"enableToggleButton")
        self.enableToggleButton.setEnabled(False)

        Form1Layout.addWidget(self.enableToggleButton,0,1)

        self.connect(
            self.enableToggleButton,
            qt.SIGNAL("clicked()"),
            self.toggleScriptEnabled
            )
        self.connect(
            self.scriptsList,
            qt.SIGNAL("pressed(QListViewItem *)"),
            self.itemSelected
            )
        self.updateToggleEnableButton()
        self.reloadState()
            
        self.resize(qt.QSize(590,653).expandedTo(self.minimumSizeHint()))
        self.clearWState(qt.Qt.WState_Polished)
    def reloadState(self):
        self.scriptsList.clear()
        self.stateChanged = False
        self.loadAvailableScripts()
        self.updateScriptInfo()
        
    def toggleScriptEnabled(self):
        item = self.scriptsList.currentItem()
        item.getParams()["enabled"] = not item.getParams()["enabled"]
        item.updateTexts()
        self.updateToggleEnableButton()
    def updateToggleEnableButton(self):
        item = self.scriptsList.currentItem()
        if item == None:
            self.enableToggleButton.setEnabled(False)
            self.enableToggleButton.setText("Enable")
            return
        self.enableToggleButton.setEnabled(True)
        if item.getParams()["enabled"]:
            self.enableToggleButton.setText("Disable")
        else:
            self.enableToggleButton.setText("Enable")
    def changed(self):
        return self.stateChanged
    def itemSelected(self, item):
        self.updateToggleEnableButton()
        self.updateScriptInfo()
        self.stateChanged = True
    def updateScriptInfo(self):
        item = self.scriptsList.currentItem()
        if item == None:
            self.scriptNameField.setText("")
            self.scriptPathField.setText("")
            self.scriptDescriptionField.setText("")
            return
        self.scriptNameField.setText(item.getParams()["name"])
        self.scriptPathField.setText(item.getParams()["path"])
        self.scriptDescriptionField.setText(item.getParams()["description"])
        
    def loadAvailableScripts(self):
        (scripts, enabledScriptsList) = getAvailableScripts("widget", "ScriptsToLoad", "scripts") 
        
        for scriptParams in scripts:
            item = ScriptListViewItem(
                scriptParams, 
                self.scriptsList
                )
    def getEnabledScripts(self):
        item = self.scriptsList.firstChild()
        enabledScripts = []
        while item != None:
            if item.getParams()["enabled"]:
                enabledScripts.append(item.getParams()["path"])
            item = item.nextSibling()
        return enabledScripts


class SingleScriptSelectionWindow(qt.QWidget):
    def __init__(self,parent = None,name = None):
        qt.QWidget.__init__(self,parent,name)

        Form1Layout = qt.QGridLayout(self,1,1,11,6,"Form1Layout")

        layout4 = qt.QVBoxLayout(None,0,6,"layout4")

        self.scriptsList = qt.QListView(self,"scriptsList")
        self.scriptsList.addColumn("Script")
        self.scriptsList.setResizePolicy(qt.QListView.AutoOneFit)
        self.scriptsList.setShowSortIndicator(1)
        layout4.addWidget(self.scriptsList)

        layout2 = qt.QHBoxLayout(None,0,6,"layout2")

        self.textLabel1 = qt.QLabel(self,"textLabel1")
        self.textLabel1.setText("Script name:")
        layout2.addWidget(self.textLabel1)

        self.scriptNameField = qt.QLabel(self,"scriptNameField")
        layout2.addWidget(self.scriptNameField)
        layout4.addLayout(layout2)

        layout2_2 = qt.QHBoxLayout(None,0,6,"layout2_2")

        self.textLabel1_2 = qt.QLabel(self,"textLabel1_2")
        self.textLabel1_2.setText("Script path:")
        layout2_2.addWidget(self.textLabel1_2)

        self.scriptPathField = qt.QLabel(self,"scriptPathField")
        layout2_2.addWidget(self.scriptPathField)
        layout4.addLayout(layout2_2)

        self.textLabel3 = qt.QLabel(self,"textLabel3")
        self.textLabel3.setText("Script description:")
        layout4.addWidget(self.textLabel3)

        self.scriptDescriptionField = qt.QTextBrowser(self,"scriptDescriptionField")
        self.scriptDescriptionField.setMinimumSize(qt.QSize(60,0))
        layout4.addWidget(self.scriptDescriptionField)

        Form1Layout.addMultiCellLayout(layout4,0,1,0,0)

        self.connect(
            self.scriptsList,
            qt.SIGNAL("pressed(QListViewItem *)"),
            self.itemSelected
            )
        self.reloadState()
        self.resize(qt.QSize(590,653).expandedTo(self.minimumSizeHint()))
        self.clearWState(qt.Qt.WState_Polished)
    def itemSelected(self, item):
        self.updateScriptInfo()
        self.stateChanged = True
    def changed(self):
        return self.stateChanged
    def updateScriptInfo(self):
        item = self.scriptsList.currentItem()
        if item == None:
            self.scriptNameField.setText("")
            self.scriptPathField.setText("")
            self.scriptDescriptionField.setText("")
            return
        self.scriptNameField.setText(item.getParams()["name"])
        self.scriptPathField.setText(item.getParams()["path"])
        self.scriptDescriptionField.setText(item.getParams()["description"])
    def reloadState(self):
        self.stateChanged = False
        self.scriptsList.clear()
        self.loadScripts()
        self.updateScriptInfo()
    def loadScripts(self):
        (scripts, enabledScriptsList) = self.fetchAvailableScripts()
        
        for scriptParams in scripts:
            item = ScriptListViewItem(
                scriptParams, 
                self.scriptsList
                )
            if item.getParams()["enabled"]:
                self.scriptsList.setSelected(item, True)

        self.updateScriptInfo()
        
    def getChosenScript(self):
        item = self.scriptsList.selectedItem()
        
        if item == None:
            return None
        return item.getParams()["path"]

class ThemeScriptsWindow(SingleScriptSelectionWindow):
    def __init__(self,parent = None,name = None):
        SingleScriptSelectionWindow.__init__(self,parent,name)
            

    def fetchAvailableScripts(self):
        (scripts, enabledScriptsList) = getAvailableScripts("theme", "ThemeManager", "scriptPath") 
        
        if len(enabledScriptsList) > 1:
            qt.QMessageBox.warning(
                None, "Too many theme managers", "You have more than one theme manager defined in your configuration. This is not valid configuration. Using first one."
                )
            enabledScriptsList = enabledScriptsList[0]
            
            firstFound = False
            for scriptParams in scripts:
                if scriptParams["enabled"]:
                    if firstFound == False:
                        firstFound = True
                    else:
                        #disabling consecutive enabled scripts, only first should be enabled
                        scriptParams["enabled"] = False
        #if len(enabledScriptsList) < 1:
            #qt.QMessageBox.warning(
                #None, "No default theme manager", "You do not have theme manager set, "
                #)
        return (scripts, enabledScriptsList)

class LoaderScriptsWindow(SingleScriptSelectionWindow):
    def __init__(self,parent = None,name = None):
        SingleScriptSelectionWindow.__init__(self,parent,name)
            

    def fetchAvailableScripts(self):
        (scripts, enabledScriptsList) = getAvailableScripts("scriptManager", "ScriptLoader", "scriptPath") 
        
        if len(enabledScriptsList) > 1:
            qt.QMessageBox.warning(
                None, "Too many script loaders", "You have more than one script loader defined in your configuration. This is not valid configuration. Using first one."
                )
            enabledScriptsList = enabledScriptsList[0]
            
            firstFound = False
            for scriptParams in scripts:
                if scriptParams["enabled"]:
                    if firstFound == False:
                        firstFound = True
                    else:
                        #disabling consecutive enabled scripts, only first should be enabled
                        scriptParams["enabled"] = False
        #if len(enabledScriptsList) < 1:
            #qt.QMessageBox.warning(
                #None, "No default theme manager", "You do not have theme manager set, "
                #)
        return (scripts, enabledScriptsList)

class DefaultScriptManagerWindow(qt.QDialog):
    def __init__(self,parent = None,name = None,modal = 0,fl = 0):
        qt.QDialog.__init__(self,parent,name,modal,fl)
        self.setCaption("Cbar configuration")
        self.mainLayout = qt.QVBoxLayout(self)
        self.tabWidget = kdeui.KJanusWidget(self, "tabWidget", kdeui.KJanusWidget.IconList)
        self.mainLayout.addWidget(self.tabWidget)
        self.pages = []
        
        page = self.tabWidget.addVBoxPage("Widgets scripts", "Cbar display and action scripts", kdecore.DesktopIcon("folder_favorite") )
        self.widgetScriptsPage = WidgetScriptsWindow(page, "widgetsScriptsWindow")
        self.pages.append(self.widgetScriptsPage)
        
        page = self.tabWidget.addVBoxPage("Theme scripts", "Cbar theme scripts", kdecore.DesktopIcon("iconthemes"))
        self.themeScriptsPage = ThemeScriptsWindow(page, "themeScriptsWindow")
        self.pages.append(self.themeScriptsPage)
        
        page = self.tabWidget.addVBoxPage("Script managers", "Cbar script managers", kdecore.DesktopIcon("folder_crystal"))
        self.scriptLoadersPage = LoaderScriptsWindow(page, "scriptManagersWindow")
        self.pages.append(self.scriptLoadersPage)
        
        layout7 = qt.QHBoxLayout(None,5,6,"layout7")
        self.installButton = qt.QPushButton(self, "installButton")
        self.installButton.setText("Install new script...")
        layout7.addWidget(self.installButton)
        
        spacer2 = qt.QSpacerItem(40,20,qt.QSizePolicy.Expanding,qt.QSizePolicy.Minimum)
        layout7.addItem(spacer2)

        self.okButton = kdeui.KPushButton(self,"okButton")
        self.okButton.setProperty("stdItem",qt.QVariant(1))
        layout7.addWidget(self.okButton)

        self.cancelButton = kdeui.KPushButton(self,"cancelButton")
        self.cancelButton.setProperty("stdItem",qt.QVariant(2))
        layout7.addWidget(self.cancelButton)

        self.mainLayout.addLayout(layout7)
        
        self.connect(
            self.okButton,
            qt.SIGNAL("clicked()"),
            self,
            qt.SLOT("accept()")
            )
        self.connect(
            self.cancelButton,
            qt.SIGNAL("clicked()"),
            self,
            qt.SLOT("reject()")
            )
        self.connect(
            self.installButton,
            qt.SIGNAL("clicked()"),
            self.installScript
            )
        self.resize(qt.QSize(700,500).expandedTo(self.minimumSizeHint()))
    def installScript(self):
        packageFileName = kfile.KFileDialog.getOpenFileName(
            "", #start dir
            "*.tgz | Cbar packages ", #filter
            self, #parent
            "Please choose package with script" #caption
            )
        if packageFileName.isEmpty():
            return
        self.installPackage(packageFileName)
        self.reloadAllPages()
    def reloadAllPages(self):
        for page in self.pages:
            page.reloadState()
    def installPackage(self, packageFileName):
        #decision = qt.QMessageBox.question(
            #self, #parent, 
            #"Where to install script", # caption
            #"Do you want to install this script for yourself or for all users (this option requires administrative password)? ", #text, 
            #"Install for yourself", #button0Text 
            #"Install for all users", # button1Text 
            #"Cancel", #button2Text 
            #0, #defaultButtonNumber = 0
            #2 #escapeButtonNumber 
            #)
        #if decision == 2:
            ##cancel
            #return
        #installGlobally = False 
        #if decision == 1:
            #installGlobally = True
        self.installPackageToLocation(packageFileName, False)
    def installPackageToLocation(self, packageFileName, installGlobally):
        import subprocess
        
        if installGlobally == True:
            qt.QMessageBox.warning(
                None, 
                "Global installation not implemented", 
                "Installation for all users has not been yet implemented."
                )
            return
        else:
            unpackDirectory = str(kdecore.locateLocal("data", "cbar/scripts/"))
            
        commands = ["mkdir", "-p", str(unpackDirectory)]
        retcode = subprocess.call(commands)
        if retcode != 0:
            qt.QMessageBox.critical(
                None, 
                "Failed to create scripts directory", 
                qt.QString("Failed to create directory for scripts (%1), error code=%2").arg(unpackDirectory).arg(retcode)
                )
            return
        
        commands = ["tar", "zxf", str(packageFileName), "-C", str(unpackDirectory)]
        print "unpackDirectory=" + str(unpackDirectory) + ", commands=" + str(commands) + "\n"
        retcode = subprocess.call(commands)
        if retcode != 0:
            qt.QMessageBox.critical(
                None, 
                "Failed to unpack script package", 
                qt.QString("Failed to unpack script package, error code=%1").arg(retcode)
                )
            return
        else:
            qt.QMessageBox.information(
                None, 
                "Script package installed", 
                "Script package has been successfully installed. You can find script in proper scripts category."
                )
            return
        return

class DefaultScriptLoader(qt.QObject):
    def __init__( self, *args ):
        apply(qt.QObject.__init__, (self,) + args)
        self.register()
        
    def openScriptManager(self):
        scriptManager = DefaultScriptManagerWindow()
        result = scriptManager.exec_loop()
        if result == qt.QDialog.Accepted:
            if scriptManager.scriptLoadersPage.changed():
                scriptLoaderScript = scriptManager.scriptLoadersPage.getChosenScript()
                self.writeScriptLoaderScript(scriptLoaderScript)
            if scriptManager.themeScriptsPage.changed():
                themeScript = scriptManager.themeScriptsPage.getChosenScript()
                self.writeThemeScript(themeScript)
            
            if scriptManager.widgetScriptsPage.changed():
                enabledScripts = scriptManager.widgetScriptsPage.getEnabledScripts()
                self.writeEnabledScripts(enabledScripts)
            
    def writeThemeScript(self, themeScript):
        global getConfig
        
        kconfig = getConfig()
        kconfig.setGroup("ThemeManager")
        oldScript = kconfig.readPathEntry("scriptPath")
        
        if qt.QString(themeScript) == qt.QString(oldScript):
            return
        
        kconfig.writePathEntry("scriptPath", themeScript)
        kconfig.sync()
        
        qt.QMessageBox.warning(
            None, 
            "Application restart needed", 
            "Theme manager cannot be changed during execution. Please restart application for the changes to make effect."
            )

            
    def writeScriptLoaderScript(self, script):
        global getConfig
        
        kconfig = getConfig()
        kconfig.setGroup("ScriptLoader")
        oldScript = kconfig.readPathEntry("scriptPath")
        
        if qt.QString(script) == qt.QString(oldScript):
            return
        
        kconfig.writePathEntry("scriptPath", script)
        kconfig.sync()
        
        qt.QMessageBox.warning(
            None, 
            "Application restart needed", 
            "Script manager cannot be changed during execution. Please restart application for the changes to make effect."
            )
            
    def writeEnabledScripts(self, enabledScripts):
        global getConfig
        global executeScript
        
        qlist = qt.QStringList()
        newEnabledSet = set(enabledScripts)
        
        for s in enabledScripts:
            qlist.append(qt.QString(s))
        
        kconfig = getConfig()
        kconfig.setGroup("ScriptsToLoad")
        oldScripts = kconfig.readPathListEntry("scripts")
        kconfig.writePathEntry("scripts", qlist)
        kconfig.sync()
        
        oldEnabledSet = set()
        
        for oldScriptPath in oldScripts:
            oldEnabledSet.add(str(oldScriptPath))
        
        for scriptPath in newEnabledSet.difference(oldEnabledSet):
            executeScript(scriptPath)
            
        if len(oldEnabledSet.difference(newEnabledSet)) != 0:
            qt.QMessageBox.warning(None, "Application restart needed", "Dynamic unloading of scripts is not yet implemented.\nPlease restart application for the changes to make effect.")
        
    def register(self):
        cbarInterface = cbar.Cbar()
        connectionObject = cbarInterface.registerScriptLoader(self)
        qt.QObject.connect(
            connectionObject,
            qt.SIGNAL("signalOpenScriptManager()"),
            self.openScriptManager
            )

def executeScript(scriptPath):
    cbarInterface = cbar.Cbar()
    cbarInterface.executeScript(scriptPath)

def loadScriptsFromConfig(kconfig):
    """Loads scripts given by config.
    NOTE: Loading script _must_ be done from toplevel of script,
    not from class, otherwise it does not work (why???).
    """
    global executeScript
    
    kconfig.setGroup("ThemeManager")
    themeManagerPath = kconfig.readPathEntry("scriptPath")
    if len(themeManagerPath) == 0:
        themeManagerPath = kdecore.locate("data", "cbar/scripts/" + "defaultThemeManager.py")
    if len(themeManagerPath) == 0:
        raise Exception, "Cannot find any theme manager"
    executeScript(str(themeManagerPath))
    
    kconfig.setGroup("ScriptsToLoad")
    scripts = kconfig.readPathListEntry("scripts") #entryName, default
    for scriptPath in scripts:
        executeScript(str(scriptPath))

scriptLoader = DefaultScriptLoader()
loadScriptsFromConfig(getConfig())


