import cbar
import qt
import kio
import kdeui
import dcopext
import kdecore

class FilemanagementActionWidget(qt.QWidget):
    def __init__( self, actionText, actionName, actionTooltip, actionConditions, *args ):
        apply(qt.QWidget.__init__, (self,) + args )
        self.actionText = actionText
        self.actionName = actionName
        
        self.supportsIsActionEnabled = actionConditions.get("supportsIsActionEnabled", False)
        self.supportsZeroItems = actionConditions.get("supportsZeroItems", False)
        self.supportsOneItem = actionConditions.get("supportsOneItem", False)
        self.supportsManyItems = actionConditions.get("supportsManyItems", False)
        
        layout = qt.QVBoxLayout(self)
        self.actionLabel = kdeui.KURLLabel(self)
        self.actionLabel.setText(actionText)
        self.actionLabel.setTipText(actionTooltip)
        self.actionLabel.setUseTips()
        self.actionLabel.setFloat(False)
        self.actionLabel.setGlow(False)
        self.actionLabel.setUnderline(True)
        layout.addWidget(self.actionLabel, 0, qt.Qt.AlignLeft | qt.Qt.AlignTop)
        qt.QObject.connect(
            self.actionLabel, 
            qt.SIGNAL("leftClickedURL(const QString&)"),
            self.clickedLabel
            )
        self.hide()
    def isActionEnabled(self):
        dcop = kdecore.KApplication.dcopClient()
        dcopObj = dcopext.DCOPObj(
            kdecore.KApplication.dcopClient().appId(), 
            dcop,
            self.topLevelWidget().name()
            )
        ok, result = dcopObj.actionIsEnabled(self.actionName)
        
        if ok == False:
            qt.QMessageBox.warning(
                None, 
                "Calling DCOP action failed", 
                u"Failed to check if Konqueror action \"%s\" is enabled  " % (self.actionName)
                )
            return False
        else:
            return result
    def itemsSelected(self, items):
        # Using actionIsEnabled() is not reliable enough to make it only check, so we are checking also number of selected items
        numItems = len(items)
        
        if (self.supportsIsActionEnabled):
            actionEnabled = self.isActionEnabled()
        else:
            #Do not check actionIsEnabled DCOP call
            actionEnabled = True

        if ((numItems == 0 and self.supportsZeroItems) or (numItems == 1 and self.supportsOneItem) or (numItems > 1 and self.supportsManyItems) ) and (actionEnabled) :
            self.show()
        else:
            self.hide()
        #qt.QMessageBox.information(self, "Debug", u"numItems=%s actionName=%s" % (numItems, self.actionName))
    def clickedLabel(self):
        dcop = kdecore.KApplication.dcopClient()
        dcopObj = dcopext.DCOPObj(
            kdecore.KApplication.dcopClient().appId(), 
            dcop,
            self.topLevelWidget().name()
            )
        ok, result = dcopObj.activateAction(self.actionName)
        #qt.QMessageBox.information(self, "Debug", u"ok=%s Result=%s actionName=%s" % (ok, result, self.actionName))

#Available actions, got by running in shell: dcop konqueror-7134 'konqueror-mainwindow#1' 'actions()'
#clear_location
#rename
#splitviewh
#go_trash
#editMimeType
#reload_all_tabs
#activate_tab_01
#forward
#usehtml
#help_about_kde
#stop
#activate_tab_02
#options_configure_extensions
#open_terminal
#new_menu
#movefiles
#help_about_app
#toolbar_url_combo
#trash
#activate_tab_03
#removeview
#configurespellcheck
#go_most_often
#go_media
#go_system
#animated_logo
#copy
#undo
#activate_tab_04
#go_autostart
#konqintro
#del
#activate_tab_05
#activatenexttab
#saveViewPropertiesLocally
#go_history
#new_window
#help_whats_this
#activate_tab_06
#newtab
#saveviewprofile
#go_url
#activate_tab_07
#go_settings
#back
#print
#activate_tab_08
#removecurrenttab
#breakoffcurrenttab
#go_network_folders
#go_applications
#duplicate_window
#location_label
#tab_move_left
#activate_tab_09
#activateprevtab
#help_contents
#reload
#tab_move_right
#removeLocalProperties
#bookmarks
#options_configure_toolbars
#link
#lock
#findfile
#properties
#devnew
#copyfiles
#removeothertabs
#up
#sendPage
#konsolepart
#help_report_bug
#paste
#cut
#duplicatecurrenttab
#sendURL
#konq_sidebartng
#options_show_menubar
#fullscreen
#saveremoveviewprofile
#activate_tab_10
#history
#konq_create_dir
#loadviewprofile
#activate_tab_11
#options_configure_keybinding
#quit
#activate_tab_12
#splitviewv
#options_configure
#home
#open_location

def createAction(actionText, actionName, actionTooltip, features):
    widget = FilemanagementActionWidget(actionText, actionName, actionTooltip, features)
    cbarInterface = cbar.Cbar()
    
    params = { "type": "Actions"}
    
    connectionObject = cbarInterface.addCbarWidget(widget, params)
    qt.QObject.connect(
        connectionObject,
        qt.SIGNAL("signalItemsSelected(const KFileItemList &)"),
        widget.itemsSelected
        )
    
    
supportsNone = {"supportsZeroItems": True}
supportsOne = {"supportsOneItem": True}
supportsMany = {"supportsOneItem": True, "supportsManyItems": True}
supportsAny = {"supportsZeroItems": True, "supportsOneItem": True, "supportsManyItems": True}
supportsAnyWithActionEnabled = {"supportsZeroItems": True, "supportsOneItem": True, "supportsManyItems": True, "supportsIsActionEnabled": True}

createAction("Copy", "copy", "Copies selected files to clipboard", supportsMany)
createAction("Paste", "paste", "Pastes items from clipboard", supportsAny)
createAction("Cut", "cut", "Copies selected files to clipboard and removes them from this place", supportsMany)
createAction("Rename", "rename", "Renames selected file", supportsOne)
createAction("Delete", "trash", "Moves selected files to trash", supportsMany)
createAction("Create directory", "konq_create_dir", "Creates directory in this directory", supportsNone)

#createAction("Stop", "stop")
#createAction("Back", "back")
#createAction("Go up", "up")
#createAction("Reload", "reload")

#createAction("Print", "print")
#createAction("Copy files", "copyfiles")
#createAction("Move files", "movefiles")

#createAction("Find file", "findfile")
#createAction("Properties", "properties")

#createAction("Open location (URL)", "open_location")

