/***************************************************************************
  Copyright:
  (C) 2006 Krzysztof Lichota <krzysiek-cbar@lichota.net>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License
 *                                                                         *
 ***************************************************************************/

#include <klibloader.h>
#include <qfile.h>
#include <kdebug.h>

extern "C"
{

void* init_libcbarkonqpluginloader()
{
    //Load real cbar konq sidebar plugin using globalLibrary, to provide
    //symbols to python libraries loaded later
    //This hack can be removed if Konqueror loads plugins using globalLibrary() 
    //instead of library()
    const char * lib_name = "libcbarkonqplugin";
    kdDebug() << "KonqCbarPlugin loader - loading real library\n";
    
    KLibLoader *loader = KLibLoader::self();

    // try to load the library
    KLibrary *lib = loader->globalLibrary(QFile::encodeName(lib_name));
    if (lib)
    {
        // get the create_ function
        QString factory("init_%1");
        void *sym = lib->symbol(QFile::encodeName(factory.arg(lib_name)));

        if (sym)
        {
            // create the module
            typedef KLibFactory* (*t_func)();
            t_func func = (t_func)sym;
            KLibFactory *factory = func();
            if( !factory )
            {
                kdDebug() << QString("The library %1 does not offer a KDE compatible factory.").arg(lib_name);
                return NULL;
            }
            
            QObject::connect(factory, SIGNAL( objectCreated( QObject * ) ),
                KLibLoader::self(), SLOT( slotObjectCreated( QObject * ) ) );
            return factory;
        }
    } 
    else 
    {
        kdWarning() << "Module " << lib_name << " doesn't specify a library!" << endl;
    }
    kdDebug() << "KonqCbarPlugin loader - library loaded\n";
    return 0;
}

}; //extern "C"
